@@ -2,7 +2,7 @@ use std::{borrow::Cow, convert::TryFrom, fmt::Display};
 
 use crate::SimpleDnsError;
 
-use super::{DnsPacketContent, MAX_CHARACTER_STRING_LENGTH};
+use super::{WireFormat, MAX_CHARACTER_STRING_LENGTH};
 
 /// CharacterString is expressed in one or two ways:
 /// - as a contiguous set of characters without interior spaces,
@@ -22,32 +22,13 @@ impl<'a> CharacterString<'a> {
     }
 
     fn internal_new(data: Cow<'a, [u8]>) -> crate::Result<Self> {
-        if data.len() > MAX_CHARACTER_STRING_LENGTH || !Self::is_valid(&data) {
+        if data.len() > MAX_CHARACTER_STRING_LENGTH {
             return Err(SimpleDnsError::InvalidCharacterString);
         }
-
         Ok(Self { data })
     }
 
-    fn is_valid(data: &'_ [u8]) -> bool {
-        if data[0] == b'"' {
-            if data[data.len() - 1] != b'"' {
-                return false;
-            }
-
-            for (p, c) in data[1..data.len() - 1].iter().enumerate() {
-                if *c == b'"' && data[p] != b'\\' {
-                    return false;
-                }
-            }
-
-            return true;
-        }
-
-        return data.iter().all(|c| *c != b' ');
-    }
-
-    /// Transforms the inner data into it's owned type
+    /// Transforms the inner data into its owned type
     pub fn into_owned<'b>(self) -> CharacterString<'b> {
         CharacterString {
             data: self.data.into_owned().into(),
@@ -55,27 +36,39 @@ impl<'a> CharacterString<'a> {
     }
 }
 
-impl<'a> DnsPacketContent<'a> for CharacterString<'a> {
-    fn parse(data: &'a [u8], position: usize) -> crate::Result<Self>
+impl<'a> TryFrom<CharacterString<'a>> for String {
+    type Error = crate::SimpleDnsError;
+
+    fn try_from(val: CharacterString<'a>) -> Result<Self, Self::Error> {
+        match String::from_utf8(val.data.into()) {
+            Ok(s) => Ok(s),
+            Err(e) => Err(SimpleDnsError::InvalidUtf8String(e)),
+        }
+    }
+}
+
+impl<'a> WireFormat<'a> for CharacterString<'a> {
+    fn parse(data: &'a [u8], position: &mut usize) -> crate::Result<Self>
     where
         Self: Sized,
     {
-        let length = data[position] as usize;
-
-        if length == 0 || Self::is_valid(&data[position + 1..position + 1 + length]) {
-            Ok(Self {
-                data: Cow::Borrowed(&data[position + 1..position + 1 + length]),
-            })
-        } else {
-            Err(SimpleDnsError::InvalidCharacterString)
+        let length = data[*position] as usize;
+        if length > MAX_CHARACTER_STRING_LENGTH || length + *position > data.len() {
+            return Err(SimpleDnsError::InvalidCharacterString);
         }
+
+        let data = &data[*position + 1..*position + 1 + length];
+        *position += length + 1;
+
+        Ok(Self {
+            data: Cow::Borrowed(data),
+        })
     }
 
-    fn append_to_vec(&self, out: &mut Vec<u8>) -> crate::Result<()> {
-        out.push(self.data.len() as u8);
-        out.extend(self.data.iter());
-
-        Ok(())
+    fn write_to<T: std::io::Write>(&self, out: &mut T) -> crate::Result<()> {
+        out.write_all(&[self.data.len() as u8])?;
+        out.write_all(&self.data)
+            .map_err(crate::SimpleDnsError::from)
     }
 
     fn len(&self) -> usize {
@@ -128,7 +121,7 @@ mod tests {
         assert!(CharacterString::new(b"Iamvalid").is_ok());
         assert!(CharacterString::new(br#""I am valid""#).is_ok());
         assert!(CharacterString::new(br#""I am \" also valid""#).is_ok());
-        assert!(CharacterString::new(b"I am invalid").is_err());
+        assert!(CharacterString::new(b"I am valid").is_ok());
 
         let long_string = [0u8; 300];
         assert!(CharacterString::new(&long_string).is_err());
@@ -136,7 +129,7 @@ mod tests {
 
     #[test]
     fn parse() {
-        let c_string = CharacterString::parse(b"\x0esome_long_text", 0);
+        let c_string = CharacterString::parse(b"\x0esome_long_text", &mut 0);
         assert!(c_string.is_ok());
         let c_string = c_string.unwrap();
         assert_eq!(15, c_string.len());
@@ -147,7 +140,7 @@ mod tests {
     fn append_to_vec() {
         let mut out = Vec::new();
         let c_string = CharacterString::new("some_long_text".as_bytes()).unwrap();
-        c_string.append_to_vec(&mut out).unwrap();
+        c_string.write_to(&mut out).unwrap();
 
         assert_eq!(b"\x0esome_long_text", &out[..]);
     }
