@@ -1,6 +1,8 @@
 use std::borrow::Cow;
 
-use crate::dns::{DnsPacketContent, MAX_NULL_LENGTH};
+use crate::dns::{WireFormat, MAX_NULL_LENGTH};
+
+use super::RR;
 
 /// NULL resources are used to represent any kind of information.
 #[derive(Debug, PartialEq, Eq, Hash, Clone)]
@@ -9,6 +11,10 @@ pub struct NULL<'a> {
     data: Cow<'a, [u8]>,
 }
 
+impl<'a> RR for NULL<'a> {
+    const TYPE_CODE: u16 = 10;
+}
+
 impl<'a> NULL<'a> {
     /// Creates a new NULL rdata
     pub fn new(data: &'a [u8]) -> crate::Result<Self> {
@@ -27,7 +33,7 @@ impl<'a> NULL<'a> {
         &self.data
     }
 
-    /// Transforms the inner data into it's owned type
+    /// Transforms the inner data into its owned type
     pub fn into_owned<'b>(self) -> NULL<'b> {
         NULL {
             length: self.length,
@@ -36,17 +42,19 @@ impl<'a> NULL<'a> {
     }
 }
 
-impl<'a> DnsPacketContent<'a> for NULL<'a> {
-    fn parse(data: &'a [u8], position: usize) -> crate::Result<Self>
+impl<'a> WireFormat<'a> for NULL<'a> {
+    fn parse(data: &'a [u8], position: &mut usize) -> crate::Result<Self>
     where
         Self: Sized,
     {
-        Self::new(&data[position..])
+        let data = &data[*position..];
+        *position += data.len();
+        Self::new(data)
     }
 
-    fn append_to_vec(&self, out: &mut Vec<u8>) -> crate::Result<()> {
-        out.extend(self.data.iter());
-        Ok(())
+    fn write_to<T: std::io::Write>(&self, out: &mut T) -> crate::Result<()> {
+        out.write_all(&self.data)
+            .map_err(crate::SimpleDnsError::from)
     }
 
     fn len(&self) -> usize {
