@@ -1,6 +1,8 @@
 use std::{collections::HashMap, convert::TryInto};
 
-use crate::dns::{DnsPacketContent, Name};
+use crate::dns::{name::Label, Name, WireFormat};
+
+use super::RR;
 
 /// SOA records are used to mark the start of a zone of authority
 #[derive(Debug, PartialEq, Eq, Hash, Clone)]
@@ -22,17 +24,12 @@ pub struct SOA<'a> {
     pub minimum: u32,
 }
 
-impl<'a> SOA<'a> {
-    fn append_commom(&self, out: &mut Vec<u8>) -> crate::Result<()> {
-        out.extend(self.serial.to_be_bytes());
-        out.extend(self.refresh.to_be_bytes());
-        out.extend(self.retry.to_be_bytes());
-        out.extend(self.expire.to_be_bytes());
-        out.extend(self.minimum.to_be_bytes());
-        Ok(())
-    }
+impl<'a> RR for SOA<'a> {
+    const TYPE_CODE: u16 = 6;
+}
 
-    /// Transforms the inner data into it's owned type
+impl<'a> SOA<'a> {
+    /// Transforms the inner data into its owned type
     pub fn into_owned<'b>(self) -> SOA<'b> {
         SOA {
             mname: self.mname.into_owned(),
@@ -44,22 +41,33 @@ impl<'a> SOA<'a> {
             minimum: self.minimum,
         }
     }
+
+    fn write_common<T: std::io::Write>(&self, out: &mut T) -> crate::Result<()> {
+        out.write_all(&self.serial.to_be_bytes())?;
+        out.write_all(&self.refresh.to_be_bytes())?;
+        out.write_all(&self.retry.to_be_bytes())?;
+        out.write_all(&self.expire.to_be_bytes())?;
+        out.write_all(&self.minimum.to_be_bytes())?;
+
+        Ok(())
+    }
 }
 
-impl<'a> DnsPacketContent<'a> for SOA<'a> {
-    fn parse(data: &'a [u8], position: usize) -> crate::Result<Self>
+impl<'a> WireFormat<'a> for SOA<'a> {
+    fn parse(data: &'a [u8], position: &mut usize) -> crate::Result<Self>
     where
         Self: Sized,
     {
         let mname = Name::parse(data, position)?;
-        let rname = Name::parse(data, position + mname.len())?;
-        let offset = mname.len() + rname.len();
+        let rname = Name::parse(data, position)?;
 
-        let serial = u32::from_be_bytes(data[offset..offset + 4].try_into()?);
-        let refresh = i32::from_be_bytes(data[offset + 4..offset + 8].try_into()?);
-        let retry = i32::from_be_bytes(data[offset + 8..offset + 12].try_into()?);
-        let expire = i32::from_be_bytes(data[offset + 12..offset + 16].try_into()?);
-        let minimum = u32::from_be_bytes(data[offset + 16..offset + 20].try_into()?);
+        let serial = u32::from_be_bytes(data[*position..*position + 4].try_into()?);
+        let refresh = i32::from_be_bytes(data[*position + 4..*position + 8].try_into()?);
+        let retry = i32::from_be_bytes(data[*position + 8..*position + 12].try_into()?);
+        let expire = i32::from_be_bytes(data[*position + 12..*position + 16].try_into()?);
+        let minimum = u32::from_be_bytes(data[*position + 16..*position + 20].try_into()?);
+
+        *position += 20;
 
         Ok(Self {
             mname,
@@ -72,29 +80,31 @@ impl<'a> DnsPacketContent<'a> for SOA<'a> {
         })
     }
 
-    fn append_to_vec(&self, out: &mut Vec<u8>) -> crate::Result<()> {
-        self.mname.append_to_vec(out)?;
-        self.rname.append_to_vec(out)?;
-        self.append_commom(out)
+    fn write_to<T: std::io::Write>(&self, out: &mut T) -> crate::Result<()> {
+        self.mname.write_to(out)?;
+        self.rname.write_to(out)?;
+        self.write_common(out)
+    }
+
+    fn write_compressed_to<T: std::io::Write + std::io::Seek>(
+        &'a self,
+        out: &mut T,
+        name_refs: &mut HashMap<&'a [Label<'a>], usize>,
+    ) -> crate::Result<()> {
+        self.mname.write_compressed_to(out, name_refs)?;
+        self.rname.write_compressed_to(out, name_refs)?;
+        self.write_common(out)
     }
 
     fn len(&self) -> usize {
         self.mname.len() + self.rname.len() + 20
     }
-
-    fn compress_append_to_vec(
-        &self,
-        out: &mut Vec<u8>,
-        name_refs: &mut HashMap<u64, usize>,
-    ) -> crate::Result<()> {
-        self.mname.compress_append_to_vec(out, name_refs)?;
-        self.rname.compress_append_to_vec(out, name_refs)?;
-        self.append_commom(out)
-    }
 }
 
 #[cfg(test)]
 mod tests {
+    use crate::{rdata::RData, ResourceRecord};
+
     use super::*;
     #[test]
     fn parse_and_write_soa() {
@@ -109,12 +119,32 @@ mod tests {
         };
 
         let mut data = Vec::new();
-        assert!(soa.append_to_vec(&mut data).is_ok());
+        assert!(soa.write_to(&mut data).is_ok());
 
-        let soa = SOA::parse(&data, 0);
+        let soa = SOA::parse(&data, &mut 0);
         assert!(soa.is_ok());
         let soa = soa.unwrap();
 
         assert_eq!(data.len(), soa.len());
     }
+
+    #[test]
+    fn parse_sample() -> Result<(), Box<dyn std::error::Error>> {
+        let sample_file = std::fs::read("samples/zonefile/SOA.sample")?;
+
+        let sample_rdata = match ResourceRecord::parse(&sample_file, &mut 0)?.rdata {
+            RData::SOA(rdata) => rdata,
+            _ => unreachable!(),
+        };
+
+        assert_eq!(sample_rdata.mname, "VENERA.sample".try_into()?);
+        assert_eq!(sample_rdata.rname, "Action\\.domains.sample".try_into()?);
+        assert_eq!(sample_rdata.serial, 20);
+        assert_eq!(sample_rdata.refresh, 7200);
+        assert_eq!(sample_rdata.retry, 600);
+        assert_eq!(sample_rdata.expire, 3600000);
+        assert_eq!(sample_rdata.minimum, 60);
+
+        Ok(())
+    }
 }
